.TH audio1371 7D "1 May 2002"
.SH NAME
audio1371 \- Sound Blaster 16 PCI, Sound Blaster PCI128 audio device driver
.SH SYNOPSIS
.nf
.B audio1371:sound,audio
.B audio1371:sound,audioctl
.fi
.SH DESCRIPTION

The
.B audio1371
driver is a STREAMS based audio device driver for the Sound Blaster 16
PCI family of audio cards. These cards use an ENSONIQ AudioPCI 97
ES1371 or compatible chipset and include an AC'97 audio codec.

Starting with version 1.2 of the driver, older SoundBlaster PCI128
cards based on ES1370 chipsets and using an Asahi Kasei AK4531 audio
codec are supported as well. 

.SH APPLICATION PROGRAMMING INTERFACE
The Sound Blaster PCI device is treated as an exclusive resource; only
one process may open the device at a time for read or write access.
However, one process may open the device read-only (for recording
audio samples), while a second process may open the device write-only
(for playing audio samples).

This driver attempts to provide an implementation of the standard Sun audio
API.


.SS Audio Data Formats

The Sound Blaster 16 PCI handles 8-bit and 16-bit samples, in
mono or stereo mode.  Audio data is sampled at rates from 4,000
to 48,000 samples per seconds. 

The Sound Blaster 16 PCI handles 8-bit samples in excess-128 format
and 16-bit samples in two's complement format (16-bit PCM).  These two
formats correspond to an audio encoding of AUDIO_ENCODING_LINEAR8 with
a precision of 8 and an audio encoding of AUDIO_ENCODING_LINEAR with
precision 16. The
.B audio1371
driver also generates and accepts the encodings AUDIO_ENCODING_ULAW,
AUDIO_ENCODING_ALAW and AUDIO_ENCODING_LINEAR, all with precision 8.
The encodings AUDIO_ENCODING_ULAW and AUDIO_ENCODING_ALAW are
converted per software into 16-bit PCM format inside the driver.
Signed linear encoding with precision 8 is converted into 8-bit
excess-128 format.

.SS Audio Ports

There are three possible output ports available for the SoundBlaster
PCI cards: AUDIO_SPEAKER, AUDIO_HEADPHONE and AUDIO_LINE_OUT.  Setting
either the AUDIO_SPEAKER or the AUDIO_HEADPHONE bit in the
.I play.port
field enables the card's speaker port.  Depending on the exact model
of the SoundBlaster PCI card, this port is intended in some cases for
unamplified speakers (SB16PCI) and in other cases for active/amplified
speakers. Consult the hardware documentation for the SoundBlaster PCI
card for more information. Setting the AUDIO_LINE_OUT bit in the
.I play.port
field selects the card's line level output port.  Output volume is
controlled by software.

The
.I record.port
field of the audio information structure allows selection of which
audio source is used for recording, and may be set to one of
AUDIO_MICROPHONE, AUDIO_LINE_IN, AUDIO_CD, AUDIO_AUX1_IN,
AUDIO_AUX2_IN or AUDIO_CODEC_LOOPB_IN.  These select input from the
microphone jack, line-level input jack, internal CD input, internal
telephone answering device, internal audio from TV tuner or internal codec
loopback respectively.

.SH FILES
.PD 0
.TP 20
.B /dev/audio
linked to /dev/sound/0
.TP
.B /dev/audioctl
linked to /dev/sound/0ctl
.TP
.B /dev/sound/0
first audio device in the system
.TP
.B /dev/sound/0ctl
audio control for the first audio device
.PD
.SH SEE ALSO
.BR audioconvert (1),
.BR ioctl (2),
.BR audio (7I),
.BR streamio (7I)

.I ftp://ftp.alsa-project.org/pub/manuals/ensoniq/
.I ftp://download.intel.com/ial/scalableplatforms/audio/ac97r21.pdf
.I AK4531A Audio CODEC with 13ch Mixer & 18bit DAC

.SH NOTES
The
.B audio1371
driver will return "SUNW,CS4231" in the 
.I name
field of the audio_device structure.  This is the name of the audio
hardware in some SPARC based systems from Sun Microsystems and is not
the correct driver name for the Sound Blaster hardware.  The wrong
name was intentionally choosen to work around problems with hardcoded
audio_device names in various audio programs (audiotool, the Java JDK,
workman, ...) that derive information about the capabilities of the
audio device from the
.I name
field of the audio_device structure.  The name "SUNW,CS4231" was
choosen because this type of SPARC hardware supports roughly the same
features as the Sound Blaster 16 PCI family of audio cards.

The
.I version
field of the audio_device structure contains the PCI revision byte of
the card as a two character hex string, a comma and the decoded
hardware name of the card. The
.I config
field contains "SB16PCI".

Older SoundBlaster PCI128 cards with an ES1370 chipset use an Asahi
Kasei AK4531 Audio CODEC which is not fully AC'97 compliant. Note that
the AK4531 does not offer separate gain controls for recording and
monitoring of the recording process.  The 
.I record.gain
field of the audio_info structure is used to control the recording gain
and maps to the CODEC's hardware gain control.  The 
.I monitor_gain
field is used as a simple toggle to enable or disable monitoring of
the recording;  a
.I monitor_gain
>= 128 enables monitoring, and a value < 128 disables it.  This
monitor_gain limitation does not apply to SoundBlaster PCI cards using
the ES1371 chipset with an AC'97 audio codec.

.SH BUGS
The
.BR mixer (7I)
interface is not yet supported.
