/*-
 * Copyright (C) 2002 Juergen Keil.
 * Copyright (C) 2002 TooLs GmbH.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by TooLs GmbH.
 * 4. The name of TooLs GmbH may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TOOLS GMBH ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL TOOLS GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/* register definitions for the "Asahi Kasei AK4531A" audio codec */

/* control register map */
#define	AK_MASTER_VOLUME_LCH	0	/* attenuation */
#define	AK_MASTER_VOLUME_RCH	1	/* attenuation */
#define	AK_VOICE_VOLUME_LCH	2	/* these are gain value ...*/
#define	AK_VOICE_VOLUME_RCH	3
#define	AK_FM_VOLUME_LCH	4
#define	AK_FM_VOLUME_RCH	5
#define	AK_CD_AUDIO_VOLUME_LCH	6
#define	AK_CD_AUDIO_VOLUME_RCH	7
#define	AK_LINE_VOLUME_LCH	8
#define	AK_LINE_VOLUME_RCH	9
#define	AK_AUX_VOLUME_LCH	0xa
#define	AK_AUX_VOLUME_RCH	0xb
#define	AK_MONO1_VOLUME		0xc
#define	AK_MONO2_VOLUME		0xd
#define	AK_MIC_VOLUME		0xe	/* ... gain */
#define	AK_MONOOUT_VOLUME	0xf	/* attenuation */
#define	AK_OUTPUT_MIXER_SW1	0x10
#define	AK_OUTPUT_MIXER_SW2	0x11
#define	AK_LCH_INPUT_MIXER_SW1	0x12
#define	AK_RCH_INPUT_MIXER_SW1	0x13
#define	AK_LCH_INPUT_MIXER_SW2	0x14
#define	AK_RCH_INPUT_MIXER_SW2	0x15
#define	AK_RESET_POWER_DOWN	0x16
#define	AK_CLOCK_SELECT		0x17
#define	AK_AD_INPUT_SELECT	0x18
#define	AK_MIC_AMP_GAIN		0x19


/* regs 0x00 .. 0x0f: gain/attenuation */
#define	AK_MUTE			0x80	/* for regs 0 - 0xf */
#define	AK_ATT_MASK		0x1f	/* for regs 0,1: 32 levels, 2dB step */
#define	AK_GAIN_MASK		0x1f	/* for regs 2 - 0xe: 32 levels, 2dB */
#define	AK_MONO_ATT_MASK	0x07	/* for reg 0xf: 8 levels, 4dB step */

/* reg 10, 11: output mixer */
#define	AK_OM_MIC		0x01
#define	AK_OM_CDR		0x02
#define	AK_OM_CDL		0x04
#define	AK_OM_LINER		0x08
#define	AK_OM_LINEL		0x10
#define	AK_OM_FMR		0x20
#define	AK_OM_FML		0x40

#define	AK_OM_MONO1		0x01
#define	AK_OM_MONO2		0x02
#define	AK_OM_VOICER		0x04
#define	AK_OM_VOICEL		0x08
#define	AK_OM_AUXR		0x10
#define	AK_OM_AUXL		0x20

/* reg 12..15: input mixer */
#define	AK_IM_MIC		0x01
#define	AK_IM_CDR		0x02
#define	AK_IM_CDL		0x04
#define	AK_IM_LINER		0x08
#define	AK_IM_LINEL		0x10
#define	AK_IM_FMR		0x20
#define	AK_IM_FML		0x40

#define	AK_IM_MONO1		0x01
#define	AK_IM_MONO2		0x02
#define	AK_IM_VOICEL		0x04
#define	AK_IM_VOICER		0x04
#define	AK_IM_AUXR		0x08
#define	AK_IM_AUXL		0x10
#define	AK_IM_TMONO2		0x20
#define	AK_IM_TMONO1		0x40
#define	AK_IM_TMIC		0x80

/* reg 16: reset & power down */
#define	AK_RST_NEG		0x01
#define	AK_PD_NEG		0x02

/* reg 17: clock select */
#define	AK_CSEL1		0x01
#define	AK_CSEL2		0x02

/* reg 18: select adc input source */
#define	AK_ADSEL_INPUT_MIXER	0x00
#define	AK_ADSEL_AIN_LR		0x01

/* reg 19: mic amp gain */
#define	AK_MGAIN_0dB		0x00
#define	AK_MGAIN_30dB		0x01

