/*-
 * Copyright (C) 2001-2003 Juergen Keil.
 * Copyright (C) 2001-2003 TooLs GmbH.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by TooLs GmbH.
 * 4. The name of TooLs GmbH may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TOOLS GMBH ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL TOOLS GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	_sys_audio_helper_h
#define	_sys_audio_helper_h

#include <sys/types.h>
#include <sys/stream.h>
#include <sys/audio.h>
#include <sys/sunddi.h>

#ifdef  __cplusplus
extern "C" {
#endif

#ifdef _KERNEL

/*
 * Internal audio interface return codes.
 */
#define AUDIO_SUCCESS                   (0)
#define AUDIO_FAILURE                   (-1)


/*
 * Miscellaneous defines.
 */
#define	AUDIO_NO_CHANNEL		(-1)


/*
 * Opaque handle used by all audio drivers to reference the audio
 * helper module.
 */
typedef struct audio_handle *audiohdl_t;


/*
 * Audio Support Module Entry Point Routines
 */
struct ah_ad_info;

audiohdl_t audio_hlp_attach(dev_info_t *dip, ddi_attach_cmd_t cmd,
    struct ah_ad_info *);
int audio_hlp_detach(audiohdl_t handle, ddi_detach_cmd_t cmd);
int audio_hlp_open(queue_t *q, dev_t *devp, int flag, int sflags,
    cred_t *credp);
int audio_hlp_close(queue_t *q, int flag, cred_t *credp);
int audio_hlp_getinfo(dev_info_t *dip, ddi_info_cmd_t infocmd,
    void *arg, void **result);


int audio_hlp_rput(queue_t *q, mblk_t *mp);
int audio_hlp_rsvc(queue_t *q);
int audio_hlp_wput(queue_t *q, mblk_t *mp);
int audio_hlp_wsvc(queue_t *q);

/*
 * Audio Support Module STREAMS Private Data Routines
 */
void audio_hlp_set_qptr(queue_t *q, dev_t dev, void *data);
void audio_hlp_free_qptr(queue_t *q);
dev_t audio_hlp_get_qptr_dev(queue_t *q);
void *audio_hlp_get_qptr_data(queue_t *q);
int audio_hlp_get_qptr_instance(queue_t *q);

/*
 * Audio Support Module Minor Routines
 */
int audio_hlp_get_max_chs(audiohdl_t handle);
int audio_hlp_get_minors_per_inst(audiohdl_t handle);
int audio_hlp_construct_minor(audiohdl_t handle, audio_device_type_e type);
int audio_hlp_devt_to_instance(dev_t dev);


/*
 * Audio Support Module Miscellaneous Routines
 */
dev_info_t *audio_hlp_get_dip(audiohdl_t handle);
void *audio_hlp_get_private(audiohdl_t handle);
void audio_hlp_set_private(audiohdl_t handle, void *private);
void audio_hlp_log(audiohdl_t handle, uint_t level, char *fmt, ...);


/*
 * ah_ad_sample_rates_t		- supported sample rates
 */
struct ah_ad_sample_rates {
	int		ad_limits;	/* 0 if sample rates not limits */
	uint_t		*ad_srs;	/* NULL term. list of sample rates */
};
typedef struct ah_ad_sample_rates ah_ad_sample_rates_t;

/* ah_ad_ample_rates.ad_limits */
#define	AD_SRS_FLAG_SR_NOT_LIMITS	0x00000000u
						/* samp rates not limits */
#define	AD_SRS_FLAG_SR_LIMITS		0x00000001u /* == MIXER_SR_LIMITS */
						/* samp rates set limits */

/*
 * ah_ad_ch_cap_t	- Audio Driver play/record capabilities
 */
struct ah_ad_ch_cap {
    ah_ad_sample_rates_t    ad_srs;	    /* compat mode sample rates */
    uint_t		   *ad_chs;	    /* list of channel types */
    int			    ad_int_rate;    /* interrupt rate */
    int			    ad_max_chs;	    /* max channels */
    size_t		    ad_bsize;	    /* buffer size */
};
typedef struct ah_ad_ch_cap ah_ad_ch_cap_t;


/*
 * ah_ad_cap_comb_t	- Audio Driver play/record capability combinations
 */
struct ah_ad_cap_comb {
    int		ad_prec;	/* the data precision */
    int		ad_enc;		/* the data encoding method */
};
typedef struct ah_ad_cap_comb ah_ad_cap_comb_t;


/*
 * ah_ad_entry - Audio Driver ops vector definition
 */
struct ah_ad_entry {
    int	    	(*ad_setup)(audiohdl_t handle, int stream, int flags);
    void	(*ad_teardown)(audiohdl_t handle, int stream, int dir);
    int		(*ad_set_config)(audiohdl_t handle, int stream,
			int command, int dir, int arg1, int arg2);
    int		(*ad_set_format)(audiohdl_t handle, int stream,
			int dir, int sample_rate, int channels,
			int precision, int encoding);
    int		(*ad_start_play)(audiohdl_t handle, int stream);
    void	(*ad_pause_play)(audiohdl_t handle, int stream);
    void	(*ad_stop_play)(audiohdl_t handle, int stream);
    int		(*ad_start_record)(audiohdl_t handle, int stream);
    void	(*ad_stop_record)(audiohdl_t handle, int stream);
    int		(*ad_ioctl)(audiohdl_t handle, int channel,
			queue_t *q, mblk_t *mp, int *error);
    int		(*ad_iocdata)(audiohdl_t handle, int channel,
			queue_t *q, mblk_t *mp, int *error);
    
    /*
     * extensions to sun's audio driver interface
     */
    int		(*ad_get_delay)(audiohdl_t handle, int stream);
    /* void	(*ad_fill_play_buffer)(audiohdl_t handle, int stream); */
};
typedef struct ah_ad_entry ah_ad_entry_t;

/* ad_set_config() and ad_set_format() stream # */
#define AD_SET_CONFIG_BOARD     (-1)    /* for the whole board */

/* ad_set_config() commands */
#define	AD_SET_GAIN		0x01	/* set input/ouput channel gain */
#define	AD_SET_GAIN_BAL		0x02	/* set input/ouput channel gain */
#define	AD_SET_PORT		0x03	/* set input/output port */
#define	AD_SET_MONITOR_GAIN	0x04	/* set monitor gain */
#define	AD_OUTPUT_MUTE		0x05	/* mute output */
#define	AD_MONO_MIC		0x06	/* set which mono microphone */
#define	AD_MIC_BOOST		0x07	/* enable/disable mic preamp */
#define	AD_BASS_BOOST		0x08	/* boost output bass */
#define	AD_MID_BOOST		0x09	/* boost output mid range */
#define	AD_TREBLE_BOOST		0x0a	/* boost output treble */
#define	AD_LOUDNESS		0x0b	/* enable/disable output loudness */
#define	AD_SET_DIAG_MODE	0x0c	/* set diagnostics mode */

/*
 * Support for custom audio driver ioctl()s.
 */
#define AD_WIOCDATA                     0       /* returned by Audio Driver */
#define AD_ACK                          1       /* with private ioctl() & */
#define AD_NACK                         2       /* iocdata() routines */


/*
 * ah_ad_info - Audio Driver configuration information structure
 */
struct ah_ad_info {
    int		    ad_int_vers;	/* Audio Driver interface version */
    audio_info_t   *ad_defaults;	/* Audio Driver audio_info_t struct */
    ah_ad_ch_cap_t  ad_play;		/* play capabilities */
    ah_ad_ch_cap_t  ad_record;		/* record capabilities */
    ah_ad_cap_comb_t *ad_play_comb;	/* list of play cap. combinations */
    ah_ad_cap_comb_t *ad_rec_comb;	/* list of rec cap. combinations */
    ah_ad_entry_t  *ad_entry;		/* Audio Driver entry points */
    audio_device_t *ad_dev_info;	/* device information */
    uint_t	    ad_diag_flags;	/* flags that specify diagnostics sup */
    uint_t	    ad_diff_flags;	/* format difference flags */
    uint_t	    ad_assist_flags;	/* audio stream assist flags */
    uint_t	    ad_misc_flags;	/* misc. flags */
    uint_t	    ad_translate_flags; /* translate flags */
    int		    ad_num_mics;	/* # of mic inputs */
    uint_t	    _xxx[4];		/* reserved for future use */
};
typedef struct ah_ad_info ah_ad_info_t;

/* ah_ad_info.ad_translate_flags */
#define	AD_MISC_8_P_TRANSLATE	0x00000001u	/* trans. signed to unsigned */
#define	AD_MISC_16_P_TRANSLATE	0x00000002u	/* trans. signed to unsigned */
#define	AD_MISC_8_R_TRANSLATE	0x00010000u	/* trans. unsigned to signed */
#define	AD_MISC_16_R_TRANSLATE	0x00020000u	/* trans. unsigned to signed */


/* ah_ad_info.ad_int_vers */
#define	AD_VERSION	AD_VERS1
#define	AD_VERS1	1



/*
 * Audio Helper Driver Device Dependent Driver Play Routines
 */
int audio_hlp_get_audio(audiohdl_t handle, void *buf, int channel, int samples);
void audio_hlp_play_shutdown(audiohdl_t handle, int channel);

/*
 * Audio Helper Driver Device Dependent Driver Record Routines
 */
void audio_hlp_send_audio(audiohdl_t handle, void *buf, int channel, int samples);

/*
 * Audio Mixer Driver Device Dependent Driver Miscellaneous Routines
 */
int audio_hlp_hw_state_change(audiohdl_t handle, int cmd, int dir, int value,
    int sleep);

/* am_hw_state_change() commands */
#define AD_HWSC_SET_GAIN_DELTA  0x01
#define AD_HWSC_SET_BAL_DELTA   0x02
#define AD_HWSC_MUTE_TOGGLE     0x03
#define AD_HWSC_ONLINE          0x04
#define AD_HWSC_OFFLINE         0x05
#define AD_HWSC_SET_GAIN_ABS    0x06
#define AD_HWSC_SET_BAL_ABS     0x07

#endif	/* _KERNEL */

#ifdef  __cplusplus
}
#endif

#endif
